using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PowerUpColliderManager : ColliderManager
{
    public PowerUpEffect powerUpEffect;
    public AudioClip tokenPickupSFX;
    private AudioSource _audioSource;
    private bool _powerUpCollected = false;

    public override void DoAwakeTasks()
    {
        GameObject playerGO = GameObject.Find("Player");
        _audioSource = playerGO.GetComponent<AudioSource>();
        base.DoAwakeTasks();
    }

    // Bool checks if the core has been collected, this is because the ships are made of multiple collision boxes and would trigger more than once.
    private bool _coreCollected = false;

    public override void ProcessCollision(CollisionDirection direction, Collision collision, bool wasChild)
    {
        GameObject collisionGO;
        if (wasChild)
        {
            collisionGO = collision.transform.parent.gameObject;
        }
        else
        {
            collisionGO = collision.gameObject;
        }

        // Check the tag of the collided object, Direction doesnt matter i assume
        switch (collisionGO.tag)
        {
            case "Player":
                PickupCore();
                break;
        }

    }

    /// <summary>
    /// Process the core pick up and destroy the core object.
    /// </summary>
    public void PickupCore()
    {
        // If the core has been collected ignore it.
        if (!_powerUpCollected)
        {
            _powerUpCollected = true;
            
            if (_audioSource != null)
            {
                _audioSource.PlayOneShot(tokenPickupSFX);
            }

            powerUpEffect.OnPickUp();

            Destroy(gameObject);
        }
    }
}
